// LogTimings.cpp : Defines the initialization routines for the DLL.
//

#include "LogTimings.h"

#if defined(OS_WIN32)
#include <windows.h>
#include <malloc.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

// constructor of your plugin class
//
LogTimings::LogTimings()
{
	firstCall = true;
	volumeNr = 0;
}

// destructor of your plugin class
//
LogTimings::~LogTimings()
{
}





bool LogTimings::initPlugin() // this function is called once after class has been created
{
	// This function call and check of return value is necessary to check whether access to TBV works!

	int ret = InitTBVAccess();

	// Never continue, if the "InitQXAccess" function fails
	//
	if(!ret)
		return false;

	// Test whether the used QX version is new enough to support the commands used in your plugin
	//
	int Major, Minor, SubMinor;
	tGetVersionOfTBV(&Major, &Minor, &SubMinor);

	// We require v3.0.0 or higher
	if( !checkVersionGreaterEqualVersion(Major, Minor, SubMinor, 3, 0, 0) )
	{
		//tMessageBox("Can not execute this plugin - Turbo-BrainVoyager version 3.0 or higher required.");
		return false;
	}

	return true;
}

// this is once called before real-time processing is started (Plugin log window is available when this fn is called)
//
bool LogTimings::executePreRun()
{
	// "tLogText()" sends a line of text to the "Log" text box in the plugin window
	tLogText(" ");
	tLogText("Plugin ready.");
	tLogText(" ");

	return true;
}

double diffclock(clock_t clock1,clock_t clock2) 
{ 
	double diffticks=clock1-clock2; 
	double diffms=(diffticks*1000)/CLOCKS_PER_SEC; 
	return diffms; 
}

// This fn is called just before analyzing the newly read in data for current volume; here you can do, for example, custom preprocessing of data (NOT yet called!)
//
bool LogTimings::executePreStep()
{	
	char InfoString[601];

	if(firstCall)
	{
		cReadVolTime = clock();
		cFullVolTime = clock();

		time_t mytime = time(0);
		sprintf(InfoString,"Plugin starts processing: %s", ctime(&mytime));

		tLogText(" ");
		tLogText(InfoString);
		tLogText(" ");
	}

	volumeNr++;
	sprintf(InfoString, "Volume: \t %i", volumeNr);
	tLogText(InfoString);
	
	clock_t currentTime=clock(); 
	if(firstCall)
		tLogText("ReadVolTime: \t unknown");
	else
	{
		double diff = double(diffclock(currentTime,cReadVolTime));
		sprintf(InfoString, "ReadVolTime: \t %0.f ms", diff);
		tLogText(InfoString);
	}

	return true;
}

// This is the most important plugin function and is called repeatedly once every time step during real-time processing
//
bool LogTimings::executePostStep()
{
	char InfoString[601];

	clock_t currentTime=clock(); 
	double diff = double(diffclock(currentTime,cFullVolTime));

	if(firstCall)
	{
		sprintf(InfoString, "FullVolTime: \t unknown + %0.f ms", diff);
		firstCall = false;
	}
	else
		sprintf(InfoString, "FullVolTime: \t %0.f ms", diff);
	tLogText(InfoString);
	cReadVolTime=clock();
	cFullVolTime=clock();
	tLogText("");
	return true;
}

// This is once called after real-time processing has been completed (end of scan)
//
bool LogTimings::executePostRun()
{
	char InfoString[601];
	
	time_t mytime = time(0);
	sprintf(InfoString,"Plugin completed processing: %s", ctime(&mytime));

	tLogText(" ");
	tLogText(InfoString);
	tLogText(" ");

	firstCall = true;
	volumeNr = 0;
	return true;
}


//
// C access functions (extern "C" in header)
//

// THIS FUNCTION CREATES YOUR PLUGIN CLASS
//
PLUGIN_ACCESS TBVPluginInterface *createPlugin() 
{
	return new LogTimings();
}

// return a name for your plugin
//
PLUGIN_ACCESS const char *getPluginName() 
{
	return "Log Processing Times";
}

// returns a descriptive string for your plugin
//   it is recommended to use a static C string to describe your plugin
//   (the potentially lengthy string needs then not be created repeatedly in successive calls)
//   Note that you may use simple HTML tags to format the string, such as <b></b>, <i></i> and <br>
//
static const char *PluginDescription = "This simple plugin logs timing information for reading and processing.<br>";

PLUGIN_ACCESS const char *getPluginDescription() 
{
	return PluginDescription;
}

// get credit for your work!
//
PLUGIN_ACCESS const char *getAuthor()
{
	return "<i>Michael Luehrs</i>, Brain Innovation B.V.";
}

// you may provide the name for a HTML file containing online help
//
PLUGIN_ACCESS const char *getHelpFile()
{
	return ""; // "LogTimingsHelp/index.html";
}

// you may provide a version string ("Major.Minor") for (updated) releases of your plugins
//
PLUGIN_ACCESS const char *getPluginVersion()
{
	return "1.0";
}


